﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.ComponentModel; //Necessary for INotifyPropertyChanged
using System.Threading;
using WiimoteLib;

namespace WPF_Wii_Binding_Library
{
    public class WPF_Wii : INotifyPropertyChanged
    {
        Mutex mu;

        Wiimote WiiControl;

        public event PropertyChangedEventHandler PropertyChanged;

        protected void Notify(string propName)
        {
            if (this.PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propName));
            }
        }

        ///<summary>
        /// Holder for ReadingWiimote
        /// </summary>

        bool readWiimote;

        public bool ReadWiimote
        {
            get { return this.readWiimote; }
            set
            {
                if (this.readWiimote == value) { return; }
                this.readWiimote = value;
                Notify("ReadWiimote");
            }
        }

        ///<summary>
        /// Holder for Canvas Data
        /// </summary>
        double canvasWidth;

        public double CanvasWidth
        {
            get { return this.canvasWidth; }
            set
            {
                if (this.canvasWidth == value) { return; }
                this.canvasWidth = value;
                Notify("CanvasWidth");
            }
        }

        double canvasHeight;

        public double CanvasHeight
        {
            get { return this.canvasHeight; }
            set
            {
                if (this.canvasHeight == value) { return; }
                this.canvasHeight = value;
                Notify("CanvasHeight");
            }
        }

        #region Target properties
        ///<summary>
        /// Target position (calculated property)
        ///</summary>

        WiimoteLib.Point targetRawPosition;

        public WiimoteLib.Point TargetRawPosition
        {
            get { return this.targetRawPosition; }
            set
            {
                if ((this.targetRawPosition.X == value.X) && (this.targetRawPosition.Y == value.Y)) { return; }
                this.targetRawPosition = value;
                Notify("TargetRawPosition");
            }
        }
        

        WiimoteLib.PointF targetPosition;

        public WiimoteLib.PointF TargetPosition
        {
            get { return this.targetPosition; }
            set
            {
                if ((this.targetPosition.X == value.X) && (this.targetPosition.Y == value.Y)) { return; }
                this.targetPosition = value;
                Notify("TargetPosition");
            }
        }

        bool targetPossible;

        public bool TargetPossible
        {
            get { return this.targetPossible; }
            set
            {
                if (this.targetPossible == value) { return; }
                this.targetPossible = value;
                Notify("TargetPossible");
            }
        }

        bool isTargetVisible;

        public bool IsTargetVisible
        {
            get { return this.isTargetVisible; }
            set
            {
                if (this.isTargetVisible == value) { return; }
                this.isTargetVisible = value;
                Notify("IsTargetVisible");
            }
        }
        #endregion

        #region Multi-Point mode
        ///<summary>
        /// Multi-Point Mode
        ///</summary>

        bool isMultiPointMode;

        public bool IsMultiPointMode
        {
            get { return this.isMultiPointMode; }
            set
            {
                if (this.isMultiPointMode == value) { return; }
                this.isMultiPointMode = value;
                Notify("IsMultiPointMode");
            }
        }
        #endregion

        #region IR1 Properties
        /// <summary>
        /// IR1 properties
        /// </summary>

        WiimoteLib.Point iR1RawPosition;

        public WiimoteLib.Point IR1RawPosition
        {
            get { return this.iR1RawPosition; }
            set
            {
                if ((this.iR1RawPosition.X == value.X) && (this.iR1RawPosition.Y == value.Y)) { return; }
                this.iR1RawPosition = value;
                Notify("IR1RawPosition");
            }
        }

        WiimoteLib.PointF iR1Position;

        public WiimoteLib.PointF IR1Position
        {
            get { return this.iR1Position; }
            set
            {
                if ((this.iR1Position.X == value.X) && (this.iR1Position.Y == value.Y)) { return; }
                this.iR1Position = value;
                Notify("IR1Position");
            }
        }

        bool IR1found;

        public bool IR1Found
        {
            get { return this.IR1found; }
            set
            {
                if (this.IR1found == value) { return; }
                this.IR1found = value;
                Notify("IR1Found");
            }
        }

        double iR1Size;

        public double IR1Size
        {
            get { return this.iR1Size; }
            set
            {
                if (this.iR1Size == value) { return; }
                this.iR1Size = value;
                Notify("IR1Size");
            }
        }

        bool isIR1Visible;

        public bool IsIR1Visible
        {
            get { return this.isIR1Visible; }
            set
            {
                if (this.isIR1Visible == value) { return; }
                this.isIR1Visible = value;
                Notify("IsIR1Visible");
            }
        }


        #endregion

        #region IR2 Properties
        /// <summary>
        /// IR2 properties
        /// </summary>

        WiimoteLib.Point iR2RawPosition;

        public WiimoteLib.Point IR2RawPosition
        {
            get { return this.iR2RawPosition; }
            set
            {
                if ((this.iR2RawPosition.X == value.X) && (this.iR2RawPosition.Y == value.Y)) { return; }
                this.iR2RawPosition = value;
                Notify("IR2RawPosition");
            }
        }

        WiimoteLib.PointF iR2Position;

        public WiimoteLib.PointF IR2Position
        {
            get { return this.iR2Position; }
            set
            {
                if ((this.iR2Position.X == value.X) && (this.iR2Position.Y == value.Y)) { return; }
                this.iR2Position = value;
                Notify("IR2Position");
            }
        }

        bool iR2Found;

        public bool IR2Found
        {
            get { return this.iR2Found; }
            set
            {
                if (this.iR2Found == value) { return; }
                this.iR2Found = value;
                Notify("IR2Found");
            }
        }

        double iR2Size;

        public double IR2Size
        {
            get { return this.iR2Size; }
            set
            {
                if (this.iR2Size == value) { return; }
                this.iR2Size = value;
                Notify("IR2Size");
            }
        }

        bool isIR2Visible;

        public bool IsIR2Visible
        {
            get { return this.isIR2Visible; }
            set
            {
                if (this.isIR2Visible == value) { return; }
                this.isIR2Visible = value;
                Notify("IsIR2Visible");
            }
        }


        #endregion

        #region MidPoint properties
        /// <summary>
        /// MidPoint properties
        /// </summary>

        WiimoteLib.Point midPointRawPosition;

        public WiimoteLib.Point MidPointRawPosition
        {
            get { return this.midPointRawPosition; }
            set
            {
                if ((this.midPointRawPosition.X == value.X) && (this.midPointRawPosition.Y == value.Y)) { return; }
                this.midPointRawPosition = value;
                Notify("MidPointRawPosition");
            }
        }

        WiimoteLib.PointF midPointPosition;

        public WiimoteLib.PointF MidPointPosition
        {
            get { return this.midPointPosition; }
            set
            {
                if ((this.midPointPosition.X == value.X) && (this.midPointPosition.Y == value.Y)) { return; }
                this.midPointPosition = value;
                Notify("MidPointPosition");
            }
        }

        bool midPointFound;

        public bool MidPointFound
        {
            get { return this.midPointFound; }
            set
            {
                if (this.midPointFound == value) { return; }
                this.midPointFound = value;
                Notify("MidPointFound");
            }
        }

        bool isMidPointVisible;

        public bool IsMidPointVisible
        {
            get { return this.isMidPointVisible; }
            set
            {
                if (this.isMidPointVisible == value) { return; }
                this.isMidPointVisible = value;
                Notify("IsMidPointVisible");
            }
        }

        #endregion

        #region IR3 Properties
        /// <summary>
        /// IR3 properties
        /// </summary>

        WiimoteLib.Point iR3RawPosition;

        public WiimoteLib.Point IR3RawPosition
        {
            get { return this.iR3RawPosition; }
            set
            {
                if ((this.iR3RawPosition.X == value.X) && (this.iR3RawPosition.Y == value.Y)) { return; }
                this.iR3RawPosition = value;
                Notify("IR3RawPosition");
            }
        }

        WiimoteLib.PointF iR3Position;

        public WiimoteLib.PointF IR3Position
        {
            get { return this.iR3Position; }
            set
            {
                if ((this.iR3Position.X == value.X) && (this.iR3Position.Y == value.Y)) { return; }
                this.iR3Position = value;
                Notify("IR3Position");
            }
        }

        bool iR3Found;

        public bool IR3Found
        {
            get { return this.iR3Found; }
            set
            {
                if (this.iR3Found == value) { return; }
                this.iR3Found = value;
                Notify("IR3Found");
            }
        }

        double iR3Size;

        public double IR3Size
        {
            get { return this.iR3Size; }
            set
            {
                if (this.iR3Size == value) { return; }
                this.iR3Size = value;
                Notify("IR3Size");
            }
        }

        bool isIR3Visible;

        public bool IsIR3Visible
        {
            get { return this.isIR3Visible; }
            set
            {
                if (this.isIR3Visible == value) { return; }
                this.isIR3Visible = value;
                Notify("IsIR3Visible");
            }
        }

        #endregion

        #region IR4 Properties
        /// <summary>
        /// IR4 properties
        /// </summary>

        WiimoteLib.Point iR4RawPosition;

        public WiimoteLib.Point IR4RawPosition
        {
            get { return this.iR4RawPosition; }
            set
            {
                if ((this.iR4RawPosition.X == value.X) && (this.iR4RawPosition.Y == value.Y)) { return; }
                this.iR4RawPosition = value;
                Notify("IR4RawPosition");
            }
        }

        WiimoteLib.PointF iR4Position;

        public WiimoteLib.PointF IR4Position
        {
            get { return this.iR4Position; }
            set
            {
                if ((this.iR4Position.X == value.X) && (this.iR4Position.Y == value.Y)) { return; }
                this.iR4Position = value;
                Notify("IR4Position");
            }
        }

        bool iIR4Found;

        public bool IR4Found
        {
            get { return this.iIR4Found; }
            set
            {
                if (this.iIR4Found == value) { return; }
                this.iIR4Found = value;
                Notify("IR4Found");
            }
        }

        double iR4Size;

        public double IR4Size
        {
            get { return this.iR4Size; }
            set
            {
                if (this.iR4Size == value) { return; }
                this.iR4Size = value;
                Notify("IR4Size");
            }
        }

        bool isIR4Visible;

        public bool IsIR4Visible
        {
            get { return this.isIR4Visible; }
            set
            {
                if (this.isIR4Visible == value) { return; }
                this.isIR4Visible = value;
                Notify("IsIR4Visible");
            }
        }
        #endregion

        #region Buttons
        /// <summary>
        /// Button Pressed properties
        /// </summary>

        bool isA_BtnPressed;

        public bool IsA_BtnPressed
        {
            get { return this.isA_BtnPressed; }
            set
            {
                if (this.isA_BtnPressed == value) { return; }
                this.isA_BtnPressed = value;
                Notify("IsA_BtnPressed");
            }
        }

        bool isB_BtnPressed;

        public bool IsB_BtnPressed
        {
            get { return this.isB_BtnPressed; }
            set
            {
                if (this.isB_BtnPressed == value) { return; }
                this.isB_BtnPressed = value;
                Notify("IsB_BtnPressed");
            }
        }

        bool isDown_BtnPressed;

        public bool IsDown_BtnPressed
        {
            get { return this.isDown_BtnPressed; }
            set
            {
                if (this.isDown_BtnPressed == value) { return; }
                this.isDown_BtnPressed = value;
                Notify("IsDown_BtnPressed");
            }
        }

        bool isUp_BtnPressed;

        public bool IsUp_BtnPressed
        {
            get { return this.isUp_BtnPressed; }
            set
            {
                if (this.isUp_BtnPressed == value) { return; }
                this.isUp_BtnPressed = value;
                Notify("IsUp_BtnPressed");
            }
        }

        bool isLeft_BtnPressed;

        public bool IsLeft_BtnPressed
        {
            get { return this.isLeft_BtnPressed; }
            set
            {
                if (this.isLeft_BtnPressed == value) { return; }
                this.isLeft_BtnPressed = value;
                Notify("IsLeft_BtnPressed");
            }
        }

        bool isRight_BtnPressed;

        public bool IsRight_BtnPressed
        {
            get { return this.isRight_BtnPressed; }
            set
            {
                if (this.isRight_BtnPressed == value) { return; }
                this.isRight_BtnPressed = value;
                Notify("IsRight_BtnPressed");
            }
        }

        bool isHome_BtnPressed;

        public bool IsHome_BtnPressed
        {
            get { return this.isHome_BtnPressed; }
            set
            {
                if (this.isHome_BtnPressed == value) { return; }
                this.isHome_BtnPressed = value;
                Notify("IsHome_BtnPressed");
            }
        }

        bool isOne_BtnPressed;

        public bool IsOne_BtnPressed
        {
            get { return this.isOne_BtnPressed; }
            set
            {
                if (this.isOne_BtnPressed == value) { return; }
                this.isOne_BtnPressed = value;
                Notify("IsOne_BtnPressed");
            }
        }

        bool isTwo_BtnPressed;

        public bool IsTwo_BtnPressed
        {
            get { return this.isTwo_BtnPressed; }
            set
            {
                if (this.isTwo_BtnPressed == value) { return; }
                this.isTwo_BtnPressed = value;
                Notify("IsTwo_BtnPressed");
            }
        }

        bool isMinus_BtnPressed;

        public bool IsMinus_BtnPressed
        {
            get { return this.isMinus_BtnPressed; }
            set
            {
                if (this.isMinus_BtnPressed == value) { return; }
                this.isMinus_BtnPressed = value;
                Notify("IsMinus_BtnPressed");
            }
        }

        bool isPlus_BtnPressed;

        public bool IsPlus_BtnPressed
        {
            get { return this.isPlus_BtnPressed; }
            set
            {
                if (this.isPlus_BtnPressed == value) { return; }
                this.isPlus_BtnPressed = value;
                Notify("IsPlus_BtnPressed");
            }
        }
        #endregion

        #region Accelerometer Properties
        /// <summary>
        /// Accelerometer properties
        /// </summary>
        /// 

        float accelX;

        public float AccelX
        {
            get { return this.accelX; }
            set
            {
                if (this.accelX == value) { return; }
                this.accelX = value;
                Notify("AccelX");
            }
        }

        float accelY;

        public float AccelY
        {
            get { return this.accelY; }
            set
            {
                if (this.accelY == value) { return; }
                this.accelY = value;
                Notify("AccelY");
            }
        }

        float accelZ;

        public float AccelZ
        {
            get { return this.accelZ; }
            set
            {
                if (this.accelZ == value) { return; }
                this.accelZ = value;
                Notify("AccelZ");
            }
        }
        #endregion

        #region LED Properties
        /// <summary>
        /// LED State Properties
        /// </summary>

        bool lED1;

        public bool LED1
        {
            get { return this.lED1; }
            set
            {
                if (this.lED1 == value) { return; }
                this.lED1 = value;
                Notify("LED1");
            }
        }

        bool lED2;

        public bool LED2
        {
            get { return this.lED2; }
            set
            {
                if (this.lED2 == value) { return; }
                this.lED2 = value;
                Notify("LED2");
            }
        }

        bool lED3;

        public bool LED3
        {
            get { return this.lED3; }
            set
            {
                if (this.lED3 == value) { return; }
                this.lED3 = value;
                Notify("LED3");
            }
        }

        bool lED4;

        public bool LED4
        {
            get { return this.lED4; }
            set
            {
                if (this.lED4 == value) { return; }
                this.lED4 = value;
                Notify("LED4");
            }
        }
        #endregion

        #region Rumble Property
        /// <summary>
        /// Rumble Property
        /// </summary>

        bool rumble;

        public bool Rumble
        {
            get { return this.rumble; }
            set
            {
                if (this.rumble == value) { return; }
                this.rumble = value;
                Notify("Rumble");
            }
        }
        #endregion

        #region Battery Property
        /// <summary>
        /// Battery Property
        /// </summary>

        int batteryLevel;

        public int BatteryLevel
        {
            get { return this.batteryLevel; }
            set
            {
                if (this.batteryLevel == value) { return; }
                this.batteryLevel = value;
                Notify("BatteryLevel");
            }
        }
        #endregion

        #region Nunchuk Properties
        /// <summary>
        /// Nunchuk properties
        /// </summary>

        bool isNunchukConnected;

        public bool IsNunchukConnected
        {
            get { return this.isNunchukConnected; }
            set
            {
                if (this.isNunchukConnected == value) { return; }
                this.isNunchukConnected = value;
                Notify("IsNunchukConnected");
            }
        }

        bool isC_BtnPressed;

        public bool IsC_BtnPressed
        {
            get { return this.isC_BtnPressed; }
            set
            {
                if (this.isC_BtnPressed == value) { return; }
                this.isC_BtnPressed = value;
                Notify("IsC_BtnPressed");
            }
        }

        bool isZ_BtnPressed;

        public bool IsZ_BtnPressed
        {
            get { return this.isZ_BtnPressed; }
            set
            {
                if (this.isZ_BtnPressed == value) { return; }
                this.isZ_BtnPressed = value;
                Notify("IsZ_BtnPressed");
            }
        }

        float joystickX;

        public float JoystickX
        {
            get { return this.joystickX; }
            set
            {
                if (this.joystickX == value) { return; }
                this.joystickX = value;
                Notify("JoystickX");
            }
        }

        float joystickY;

        public float JoystickY
        {
            get { return this.joystickY; }
            set
            {
                if (this.joystickY == value) { return; }
                this.joystickY = value;
                Notify("JoystickY");
            }
        }

        float nunchukAccelX;

        public float NunchukAccelX
        {
            get { return this.nunchukAccelX; }
            set
            {
                if (this.nunchukAccelX == value) { return; }
                this.nunchukAccelX = value;
                Notify("NunchukAccelX");
            }
        }

        float nunchukAccelY;

        public float NunchukAccelY
        {
            get { return this.nunchukAccelY; }
            set
            {
                if (this.nunchukAccelY == value) { return; }
                this.nunchukAccelY = value;
                Notify("NunchukAccelY");
            }
        }

        float nunchukAccelZ;

        public float NunchukAccelZ
        {
            get { return this.nunchukAccelZ; }
            set
            {
                if (this.nunchukAccelZ == value) { return; }
                this.nunchukAccelZ = value;
                Notify("NunchukAccelZ");
            }
        }
        #endregion

        #region Wii Fit Balance Board Properties

        float fitTopLeft;

        public float FitTopLeft
        {
            get { return this.fitTopLeft; }
            set
            {
                if (this.fitTopLeft == value) { return; }
                this.fitTopLeft = value;
                Notify("FitTopLeft");
            }
        }

        float fitTopRight;

        public float FitTopRight
        {
            get { return this.fitTopRight; }
            set
            {
                if (this.fitTopRight == value) { return; }
                this.fitTopRight = value;
                Notify("FitTopRight");
            }
        }

        float fitBottomLeft;

        public float FitBottomLeft
        {
            get { return this.fitBottomLeft; }
            set
            {
                if (this.fitBottomLeft == value) { return; }
                this.fitBottomLeft = value;
                Notify("FitBottomLeft");
            }
        }

        float fitBottomRight;

        public float FitBottomRight
        {
            get { return this.fitBottomRight; }
            set
            {
                if (this.fitBottomRight == value) { return; }
                this.fitBottomRight = value;
                Notify("FitBottomRight");
            }
        }

        float fitWeight;

        public float FitWeight
        {
            get { return this.fitWeight; }
            set
            {
                if (this.fitWeight == value) { return; }
                this.fitWeight = value;
                Notify("FitWeight");
            }
        }

        #endregion

        void WiiControl_WiimoteChanged(object sender, WiimoteChangedEventArgs e)
        {
            //Blocks the current thread so that we can do our dirty work
            mu.WaitOne();

            //A snapshot of the Wiimote
            WiimoteState WiiState = e.WiimoteState;

            ///////////////////////////////////////////////////////////////////////
            //Getting the Infrared Camera information into our WPF-ized properties
            //  OneWay binding only

            //IR Sensor 1
            IR1Found = WiiState.IRState.IRSensors[0].Found;
            IR1Size = WiiState.IRState.IRSensors[0].Size;
            IR1RawPosition = WiiState.IRState.IRSensors[0].RawPosition;
            iR1Position.X = WiiState.IRState.IRSensors[0].Position.X;
            iR1Position.Y = (int)WiiState.IRState.IRSensors[0].Position.Y;
            IR1Position = iR1Position;

            //IR Sensor 2

            IR2Found = WiiState.IRState.IRSensors[1].Found;
            IR2Size = WiiState.IRState.IRSensors[1].Size;
            IR2RawPosition = WiiState.IRState.IRSensors[1].RawPosition;
            IR2Position = WiiState.IRState.IRSensors[1].Position;

            //IR Sensor 3
            IR3Found = WiiState.IRState.IRSensors[2].Found;
            IR3Size = WiiState.IRState.IRSensors[2].Size;
            IR3RawPosition = WiiState.IRState.IRSensors[2].RawPosition;
            iR3Position = WiiState.IRState.IRSensors[2].Position;
            IR3Position = iR3Position;

            //IR Sensor 4
            IR4Found = WiiState.IRState.IRSensors[3].Found;
            IR4Size = WiiState.IRState.IRSensors[3].Size;
            IR4RawPosition = WiiState.IRState.IRSensors[3].RawPosition;
            iR4Position = WiiState.IRState.IRSensors[3].Position;
            IR4Position = iR4Position;

            //MidPoint

            MidPointRawPosition = WiiState.IRState.RawMidpoint;
            MidPointPosition = MultiPointConverter(WiiState.IRState.Midpoint);
            //new PointF(  (Convert.ToDouble(WiiState.IRState.MidX), Convert.ToDouble(WiiState.IRState.MidY)));

            if (IR1Found && IR2Found)
            {
                MidPointFound = true;
                TargetPossible = true;
            }
            else
            {
                MidPointFound = false;
                TargetPossible = false;
            }

            //////////////////////////////////////////////////////////////////////
            //Getting Target Position (calculated property)
            //  OneWay binding only
            WiimoteLib.PointF forTRawPos = new WiimoteLib.PointF();
            forTRawPos.X = (float)(1023.0 - MidPointRawPosition.X);
            forTRawPos.X = (float)(MidPointRawPosition.Y);
            forTRawPos = MultiPointRawConverter(forTRawPos);
            WiimoteLib.Point secondaryHolder = new WiimoteLib.Point();
            secondaryHolder.X = Convert.ToInt32(forTRawPos.X);
            secondaryHolder.Y = Convert.ToInt32(forTRawPos.Y);
            TargetRawPosition = secondaryHolder;

            WiimoteLib.PointF forTPos = new WiimoteLib.PointF();
            forTPos.X = (float)(1.0 - MidPointPosition.X);
            forTPos.X = (float)(MidPointPosition.Y);
            TargetPosition = MultiPointConverter(forTPos);

            ///////////////////////////////////////////////////////////////////////
            // Getting Button State information into our WPF-ized properties
            //  OneWay binding only

            IsA_BtnPressed = WiiState.ButtonState.A;
            IsB_BtnPressed = WiiState.ButtonState.B;
            IsDown_BtnPressed = WiiState.ButtonState.Down;
            IsLeft_BtnPressed = WiiState.ButtonState.Left;
            IsHome_BtnPressed = WiiState.ButtonState.Home;
            IsOne_BtnPressed = WiiState.ButtonState.One;
            IsRight_BtnPressed = WiiState.ButtonState.Right;
            IsTwo_BtnPressed = WiiState.ButtonState.Two;
            IsUp_BtnPressed = WiiState.ButtonState.Up;
            IsMinus_BtnPressed = WiiState.ButtonState.Minus;
            IsPlus_BtnPressed = WiiState.ButtonState.Plus;

            ///////////////////////////////////////////////////////////////////////
            // Getting Accelerometer State information into our WPF-ized properties
            //  OneWay binding only

            AccelX = WiiState.AccelState.Values.X;
            AccelY = WiiState.AccelState.Values.Y;
            AccelZ = WiiState.AccelState.Values.Z;

            ///////////////////////////////////////////////////////////////////////
            // Getting Battery State information into our WPF-ized properties
            //  OneWay binding only

            BatteryLevel = Convert.ToInt16(WiiState.Battery);

            ///////////////////////////////////////////////////////////////////////
            // Getting and Setting LED State information into our WPF-ized properties
            //  Two Way binding

            if ((LED1 != WiiState.LEDState.LED1) ||
                (LED2 != WiiState.LEDState.LED2) ||
                (LED3 != WiiState.LEDState.LED3) ||
                (LED4 != WiiState.LEDState.LED4))
            {
                WiiControl.SetLEDs(LED1, LED2, LED3, LED4);
            }

            ///////////////////////////////////////////////////////////////////////
            // Setting Rumble State information into our WPF-ized properties
            //  Two Way binding

            if (WiiState.Rumble != Rumble)
            {
                WiiControl.SetRumble(Rumble);
            }


            ///////////////////////////////////////////////////////////////////////
            // Getting Nunchuk  State information into our WPF-ized properties
            //  OneWay binding only

            if (Convert.ToString(WiiState.ExtensionType) == "Nunchuk")
            {
                IsNunchukConnected = true;
            }
            else
            {
                IsNunchukConnected = false;
            }




            //////////////////////////////////////////////////////////////////////
            // In the interest of creating a lean, mean Wii-to-WPF library, I'm 
            //  leaving out the following properties that are possibly 
            //  available via Brain Peek's work:
            //      - Accelerometer Calibration Info
            //      - Classic Controller Button States
            //      - Classic Controller Calibration Info
            //      - Classic Controller State Info
            //      - Nunchuk Callibration Info
            //  If anyone really wants this information, please contact me at 
            //      matthias.shapiro@gmail.com
            //      and I'll put it in.

            switch (WiiState.ExtensionType)
            {
                case ExtensionType.Nunchuk:

                    NunchukAccelX = WiiState.NunchukState.AccelState.Values.X;
                    NunchukAccelY = WiiState.NunchukState.AccelState.Values.Y;
                    NunchukAccelZ = WiiState.NunchukState.AccelState.Values.Z;

                    IsC_BtnPressed = WiiState.NunchukState.C;
                    IsZ_BtnPressed = WiiState.NunchukState.Z;

                    JoystickX = WiiState.NunchukState.Joystick.X;
                    JoystickY = WiiState.NunchukState.Joystick.Y;
                  
                    break;

                case ExtensionType.ClassicController:
                    /*ws.ClassicControllerState.ButtonState.A);
                    ws.ClassicControllerState.ButtonState.B);
                    ws.ClassicControllerState.ButtonState.X);
                    ws.ClassicControllerState.ButtonState.Y);
                    ws.ClassicControllerState.ButtonState.Minus);
                    ws.ClassicControllerState.ButtonState.Home);
                    ws.ClassicControllerState.ButtonState.Plus);
                    ws.ClassicControllerState.ButtonState.Up);
                    ws.ClassicControllerState.ButtonState.Down);
                    ws.ClassicControllerState.ButtonState.Left);
                    ws.ClassicControllerState.ButtonState.Right);
                    ws.ClassicControllerState.ButtonState.ZL);
                    ws.ClassicControllerState.ButtonState.ZR);
                    ws.ClassicControllerState.ButtonState.TriggerL);
                    ws.ClassicControllerState.ButtonState.TriggerR);

                    ws.ClassicControllerState.JoystickL.ToString();
                    WiiState.ClassicControllerState.JoystickR.ToString();

                    ws.ClassicControllerState.TriggerL.ToString();
                    ws.ClassicControllerState.TriggerR.ToString();*/
                    break;

                case ExtensionType.Guitar:
                    /*ws.GuitarState.FretButtonState.Green);
                    ws.GuitarState.FretButtonState.Red);
                    ws.GuitarState.FretButtonState.Yellow);
                    ws.GuitarState.FretButtonState.Blue);
                    ws.GuitarState.FretButtonState.Orange);
                    ws.GuitarState.ButtonState.Minus);
                    ws.GuitarState.ButtonState.Plus);
                    ws.GuitarState.ButtonState.StrumUp);
                    ws.GuitarState.ButtonState.StrumDown);

                    ws.GuitarState.TouchbarState.Green);
                    ws.GuitarState.TouchbarState.Red);
                    ws.GuitarState.TouchbarState.Yellow);
                    ws.GuitarState.TouchbarState.Blue);
                    ws.GuitarState.TouchbarState.Orange);

                    ws.GuitarState.Joystick.ToString();
                    ws.GuitarState.WhammyBar.ToString();
                    ws.GuitarState.GuitarType.ToString();*/
                    break;

                case ExtensionType.Drums:
                    /*ws.DrumsState.Red);
                    ws.DrumsState.Blue);
                    ws.DrumsState.Green);
                    ws.DrumsState.Yellow);
                    ws.DrumsState.Orange);
                    ws.DrumsState.Pedal);
                    ws.DrumsState.Minus);
                    ws.DrumsState.Plus);

                    ws.DrumsState.RedVelocity);
                    ws.DrumsState.BlueVelocity);
                    ws.DrumsState.GreenVelocity);
                    ws.DrumsState.YellowVelocity);
                    ws.DrumsState.OrangeVelocity);
                    ws.DrumsState.PedalVelocity);

                    ws.DrumsState.Joystick.ToString();*/
                    break;

                case ExtensionType.BalanceBoard:

                    FitTopLeft = WiiState.BalanceBoardState.SensorValuesLb.TopLeft;
                    FitTopRight = WiiState.BalanceBoardState.SensorValuesLb.TopRight;
                    FitBottomLeft = WiiState.BalanceBoardState.SensorValuesLb.BottomLeft;
                    FitBottomRight = WiiState.BalanceBoardState.SensorValuesLb.BottomRight;
                    FitWeight = WiiState.BalanceBoardState.WeightLb;
                
                    /*ws.BalanceBoardState.SensorValuesKg.TopLeft.ToString();
                    ws.BalanceBoardState.SensorValuesKg.TopRight.ToString();
                    ws.BalanceBoardState.SensorValuesKg.BottomLeft.ToString();
                    ws.BalanceBoardState.SensorValuesKg.BottomRight.ToString();
                    ws.BalanceBoardState.WeightKg.ToString();*/

                    break;
            }
        }

        public void AlignLEDs()
        {
            LED1 = true;
            LED2 = false;
            LED3 = false;
            LED4 = false;
        }

        public WiimoteLib.PointF MultiPointRawConverter(WiimoteLib.PointF InitialPos)
        {
            if (IsMultiPointMode)
            {
                float convertedX = (float)(1023.0 - InitialPos.X);
                float convertedY = (float)(767.0 - InitialPos.Y);
                InitialPos = new WiimoteLib.PointF();
                InitialPos.X = convertedX;
                InitialPos.Y = convertedY;
            }

            return InitialPos;
        }

        public WiimoteLib.PointF MultiPointConverter(WiimoteLib.PointF InitialPos)
        {
            if (IsMultiPointMode)
            {
                float convertedX = (float)(1.0 - InitialPos.X);
                float convertedY = (float)(1.0 - InitialPos.Y);
                //(convertedX, convertedY);
                InitialPos = new WiimoteLib.PointF();
                InitialPos.X = convertedX;
                InitialPos.Y = convertedY;
            }

            return InitialPos;
        }

        public void startWiiReading()
        {
            this.WiiControl = new Wiimote();

            this.WiiControl = new Wiimote();
            WiiControl.WiimoteChanged += new EventHandler<WiimoteChangedEventArgs>(WiiControl_WiimoteChanged);

            //new WiimoteChangedEventHandler(wii_OnWiimoteChanged);

            WiiControl.Connect();
            WiiControl.SetReportType(InputReport.IRAccel, true);
            AlignLEDs();
            ReadWiimote = true;
        }

        public void stopWiiReading()
        {
            this.WiiControl.Disconnect();
            ReadWiimote = false;
        }


        public WPF_Wii()
        {
            this.mu = new Mutex();

            if (ReadWiimote)
            {
                startWiiReading();
            }
        }

    }
}
